{% extends "admin/base_site.html" %}
{% load i18n %}

{% block breadcrumbs %}
<div class="breadcrumbs">
<a href="{% url 'admin:index' %}">{% translate 'Home' %}</a>
&rsaquo; {% translate 'Documentation' %}
</div>
{% endblock %}
{% block title %}{% translate 'Please install docutils' %}{% endblock %}

{% block content %}

<h1>{% translate 'Documentation' %}</h1>

<div id="content-main">
  <h3>{% blocktranslate with "https://docutils.sourceforge.io/" as link %}The admin documentation system requires Python’s <a href="{{ link }}">docutils</a> library.{% endblocktranslate %}</h3>

  <p>{% blocktranslate with "https://docutils.sourceforge.io/" as link %}Please ask your administrators to install <a href="{{ link }}">docutils</a>.{% endblocktranslate %}</p>
</div>

{% endblock %}
